@@ -7,7 +7,7 @@
  * PHP versions 4 and 5
  *
  * <LICENSE>
- * Copyright (c) 2005, Alexander Wirtz
+ * Copyright (c) 2005-2011, Alexander Wirtz
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,9 @@
  * @category    Web Services
  * @package     Services_Weather
  * @author      Alexander Wirtz <alex@pc4p.net>
- * @copyright   2005 Alexander Wirtz
+ * @copyright   2005-2011 Alexander Wirtz
  * @license     http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version     CVS: $Id: Ejse.php,v 1.23 2005/11/13 21:56:16 eru Exp $
+ * @version     CVS: $Id$
  * @link        http://pear.php.net/package/Services_Weather
  * @link        http://www.ejse.com/services/weather_xml_web_services.htm
  * @example     examples/ejse-basic.php             ejse-basic.php
@@ -62,9 +62,9 @@ require_once "Services/Weather/Common.php";
  * @category    Web Services
  * @package     Services_Weather
  * @author      Alexander Wirtz <alex@pc4p.net>
- * @copyright   2005 Alexander Wirtz
+ * @copyright   2005-2011 Alexander Wirtz
  * @license     http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version     Release: 1.4.0
+ * @version     Release: 1.4.7
  * @link        http://pear.php.net/package/Services_Weather
  * @link        http://www.ejse.com/services/weather_xml_web_services.htm
  * @example     examples/ejse-basic.php             ejse-basic.php
@@ -72,6 +72,22 @@ require_once "Services/Weather/Common.php";
 class Services_Weather_Ejse extends Services_Weather_Common {
 
     // {{{ properties
+    /**
+     * Username at ejse.com
+     *
+     * @var     string                      $_username
+     * @access  private
+     */
+    var $_username = "";
+
+    /**
+     * Password key at ejse.com
+     *
+     * @var     string                      $_password
+     * @access  private
+     */
+    var $_password = "";
+
     /**
      * WSDL object, provided by EJSE
      *
@@ -123,12 +139,14 @@ class Services_Weather_Ejse extends Services_Weather_Common {
         include_once "SOAP/Client.php";
         $this->_wsdl = new SOAP_WSDL("http://www.ejse.com/WeatherService/Service.asmx?WSDL", $this->_httpOptions);
         if (isset($this->_wsdl->fault) && Services_Weather::isError($this->_wsdl->fault)) {
-            return Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
+            $error = Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
+            return $error;
         }
 
         eval($this->_wsdl->generateAllProxies());
         if (!class_exists("WebService_Service_ServiceSoap")) {
-            return Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
+            $error = Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
+            return $error;
         }
         $this->_weatherSoap = &new WebService_Service_ServiceSoap;
 
@@ -136,6 +154,26 @@ class Services_Weather_Ejse extends Services_Weather_Common {
     }
     // }}}
 
+    // {{{ setAccountData()
+    /**
+     * Sets the neccessary account-information for ejse.com, you'll
+     * receive them after registering for the service
+     *
+     * @param   string                      $username
+     * @param   string                      $password
+     * @access  public
+     */
+    function setAccountData($username, $password)
+    {
+        if (strlen($username)) {
+            $this->_username  = $username;
+        }
+        if (strlen($password) && ctype_alnum($password)) {
+            $this->_password = $password;
+        }
+    }
+    // }}}
+
     // {{{ _checkLocationID()
     /**
      * Checks the id for valid values and thus prevents silly requests to EJSE server
@@ -210,7 +248,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
 
         $locationReturn = array();
 
-        if ($this->_cacheEnabled && ($weather = $this->_cache->get($id, "weather"))) {
+        if ($this->_cacheEnabled && ($weather = $this->_getCache($id, "weather"))) {
             // Get data from cache
             $this->_weather = $weather;
             $locationReturn["cache"] = "HIT";
@@ -223,7 +261,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
                 }
             }
 
-            $weather = $this->_weatherSoap->getWeatherInfo($id);
+            $weather = $this->_weatherSoap->getWeatherInfo2($this->_username, $this->_password, $id);
 
             if (Services_Weather::isError($weather)) {
                 return $weather;
@@ -233,8 +271,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
 
             if ($this->_cacheEnabled) {
                 // ...and cache it
-                $expire = constant("SERVICES_WEATHER_EXPIRES_WEATHER");
-                $this->_cache->extSave($id, $this->_weather, "", $expire, "weather");
+                $this->_saveCache($id, $this->_weather, "", "weather");
             }
             $locationReturn["cache"] = "MISS";
         }
@@ -266,7 +303,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
         $units    = $this->getUnitsFormat($unitsFormat);
 
         $weatherReturn = array();
-        if ($this->_cacheEnabled && ($weather = $this->_cache->get($id, "weather"))) {
+        if ($this->_cacheEnabled && ($weather = $this->_getCache($id, "weather"))) {
             // Same procedure...
             $this->_weather = $weather;
             $weatherReturn["cache"] = "HIT";
@@ -280,7 +317,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
             }
 
             // ...as last function
-            $weather = $this->_weatherSoap->getWeatherInfo($id);
+            $weather = $this->_weatherSoap->getWeatherInfo2($this->_username, $this->_password, $id);
 
             if (Services_Weather::isError($weather)) {
                 return $weather;
@@ -290,8 +327,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
 
             if ($this->_cacheEnabled) {
                 // ...and cache it
-                $expire = constant("SERVICES_WEATHER_EXPIRES_WEATHER");
-                $this->_cache->extSave($id, $this->_weather, "", $expire, "weather");
+                $this->_saveCache($id, $this->_weather, "", "weather");
             }
             $weatherReturn["cache"] = "MISS";
         }
@@ -405,7 +441,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
         $units    = $this->getUnitsFormat($unitsFormat);
 
         $forecastReturn = array();
-        if ($this->_cacheEnabled && ($forecast = $this->_cache->get($id, "forecast"))) {
+        if ($this->_cacheEnabled && ($forecast = $this->_getCache($id, "forecast"))) {
             // Same procedure...
             $this->_forecast = $forecast;
             $forecastReturn["cache"] = "HIT";
@@ -419,7 +455,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
             }
 
             // ...as last function
-            $forecast = $this->_weatherSoap->GetNineDayForecastInfo($id);
+            $forecast = $this->_weatherSoap->GetNineDayForecastInfo2($this->_username, $this->_password, $id);
 
             if (Services_Weather::isError($forecast)) {
                 return $forecast;
@@ -429,8 +465,7 @@ class Services_Weather_Ejse extends Services_Weather_Common {
 
             if ($this->_cacheEnabled) {
                 // ...and cache it
-                $expire = constant("SERVICES_WEATHER_EXPIRES_FORECAST");
-                $this->_cache->extSave($id, $this->_forecast, "", $expire, "forecast");
+                $this->_saveCache($id, $this->_forecast, "", "forecast");
             }
             $forecastReturn["cache"] = "MISS";
         }
