@@ -7,7 +7,7 @@
  * PHP versions 4 and 5
  *
  * <LICENSE>
- * Copyright (c) 2005, Alexander Wirtz
+ * Copyright (c) 2005-2011, Alexander Wirtz
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,9 @@
  * @category    Web Services
  * @package     Services_Weather
  * @author      Alexander Wirtz <alex@pc4p.net>
- * @copyright   2005 Alexander Wirtz
+ * @copyright   2005-2011 Alexander Wirtz
  * @license     http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version     CVS: $Id: Globalweather.php,v 1.34 2005/11/13 21:56:16 eru Exp $
+ * @version     CVS: $Id$
  * @link        http://pear.php.net/package/Services_Weather
  * @link        http://www.capescience.com/webservices/globalweather/index.shtml
  * @example     examples/globalweather-basic.php    globalweather-basic.php
@@ -68,9 +68,9 @@ require_once "Services/Weather/Common.php";
  * @category    Web Services
  * @package     Services_Weather
  * @author      Alexander Wirtz <alex@pc4p.net>
- * @copyright   2005 Alexander Wirtz
+ * @copyright   2005-2011 Alexander Wirtz
  * @license     http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version     Release: 1.4.0
+ * @version     Release: 1.4.7
  * @link        http://pear.php.net/package/Services_Weather
  * @link        http://www.capescience.com/webservices/globalweather/index.shtml
  * @example     examples/globalweather-basic.php    globalweather-basic.php
@@ -136,16 +136,16 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
     function _connectServer()
     {
         include_once "SOAP/Client.php";
-        $this->_wsdl = new SOAP_WSDL("http://live.capescience.com/wsdl/GlobalWeather.wsdl", $this->_httpOptions);
+        $this->_wsdl = new SOAP_WSDL("http://live.capeclear.com/ccx/GlobalWeather?wsdl", $this->_httpOptions);
         if (isset($this->_wsdl->fault) && Services_Weather::isError($this->_wsdl->fault)) {
             $error = Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
-            return;
+            return $error;
         }
 
         eval($this->_wsdl->generateAllProxies());
         if (!class_exists("WebService_GlobalWeather_StationInfo") || !class_exists("WebService_GlobalWeather_GlobalWeather")) {
             $error = Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
-            return;
+            return $error;
         }
 
         $this->_stationSoap = &new WebService_GlobalWeather_StationInfo;
@@ -168,6 +168,14 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
      */
     function _checkLocationID($id)
     {
+        // Check, if the stationSoap-Object is present. If not, connect to the Server and retrieve the WDSL data
+        if (!$this->_stationSoap) {
+            $status = $this->_connectServer();
+            if (Services_Weather::isError($status)) {
+                return $status;
+            }
+        }
+
         if (is_array($id) || is_object($id) || !strlen($id)) {
             return Services_Weather::raiseError(SERVICES_WEATHER_ERROR_NO_LOCATION, __FILE__, __LINE__);
         } elseif ($this->_stationSoap->isValidCode($id) === false) {
@@ -237,15 +245,6 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
      */
     function searchLocationByCountry($country = "")
     {
-        // Return the available countries as no country was given
-        if (!strlen($country)) {
-            $countries = $this->_stationSoap->listCountries();
-            if (Services_Weather::isError($countries)) {
-                return Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
-            }
-            return $countries;
-        }
-
         // Check, if the stationSoap-Object is present. If not, connect to the Server and retrieve the WDSL data
         if (!$this->_stationSoap) {
             $status = $this->_connectServer();
@@ -254,6 +253,15 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
             }
         }
 
+        // Return the available countries as no country was given
+        if (!strlen($country)) {
+            $countries = $this->_stationSoap->listCountries("");
+            if (Services_Weather::isError($countries)) {
+                return Services_Weather::raiseError(SERVICES_WEATHER_ERROR_WRONG_SERVER_DATA, __FILE__, __LINE__);
+            }
+            return $countries;
+        }
+
         // Now for the real search
         $countryLocs = $this->_stationSoap->searchByCountry($country);
         // Check result for validity
@@ -293,7 +301,7 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
 
         $locationReturn = array();
 
-        if ($this->_cacheEnabled && ($location = $this->_cache->get("GW-".$id, "location"))) {
+        if ($this->_cacheEnabled && ($location = $this->_getCache("GW-".$id, "location"))) {
             // Get data from cache
             $this->_location = $location;
             $locationReturn["cache"] = "HIT";
@@ -316,8 +324,7 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
 
             if ($this->_cacheEnabled) {
                 // ...and cache it
-                $expire = constant("SERVICES_WEATHER_EXPIRES_LOCATION");
-                $this->_cache->extSave("GW-".$id, $this->_location, "", $expire, "location");
+                $this->_saveCache("GW-".$id, $this->_location, "", "location");
             }
             $locationReturn["cache"] = "MISS";
         }
@@ -372,7 +379,7 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
         $units    = $this->getUnitsFormat($unitsFormat);
 
         $weatherReturn = array();
-        if ($this->_cacheEnabled && ($weather = $this->_cache->get("GW-".$id, "weather"))) {
+        if ($this->_cacheEnabled && ($weather = $this->_getCache("GW-".$id, "weather"))) {
             // Same procedure...
             $this->_weather = $weather;
             $weatherReturn["cache"] = "HIT";
@@ -396,8 +403,7 @@ class Services_Weather_Globalweather extends Services_Weather_Common {
 
             if ($this->_cacheEnabled) {
                 // ...and cache it
-                $expire = constant("SERVICES_WEATHER_EXPIRES_WEATHER");
-                $this->_cache->extSave("GW-".$id, $this->_weather, "", $expire, "weather");
+                $this->_saveCache("GW-".$id, $this->_weather, "", "weather");
             }
             $weatherReturn["cache"] = "MISS";
         }
